@@ -1,7 +1,7 @@
 /**************************************************************************
  *   nano.c  --  This file is part of GNU nano.                           *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2020 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2021 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014-2020 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
@@ -62,10 +62,6 @@ static struct termios original_state;
 
 static struct sigaction oldaction, newaction;
 		/* Containers for the original and the temporary handler for SIGINT. */
-#ifdef USE_SLANG
-static bool selfinduced = FALSE;
-		/* Whether a suspension was caused from inside nano or from outside. */
-#endif
 
 /* Create a new linestruct node.  Note that we do not set prevnode->next. */
 linestruct *make_new_node(linestruct *prevnode)
@@ -80,7 +76,6 @@ linestruct *make_new_node(linestruct *prevnode)
 #endif
 	newnode->lineno = (prevnode) ? prevnode->lineno + 1 : 1;
 #ifndef NANO_TINY
-	newnode->extrarows = -2;  /* Bad value, to make it easier to find bugs. */
 	newnode->has_anchor = FALSE;
 #endif
 
@@ -104,6 +99,9 @@ void splice_node(linestruct *afterthis, linestruct *newnode)
 /* Free the data structures in the given node. */
 void delete_node(linestruct *line)
 {
+	/* If the first line on the screen gets deleted, step one back. */
+	if (line == openfile->edittop)
+		openfile->edittop = line->prev;
 #ifdef ENABLE_WRAPPING
 	/* If the spill-over line for hard-wrapping is deleted... */
 	if (line == openfile->spillage_line)
@@ -156,7 +154,6 @@ linestruct *copy_node(const linestruct *src)
 #endif
 	dst->lineno = src->lineno;
 #ifndef NANO_TINY
-	dst->extrarows = src->extrarows;
 	dst->has_anchor = FALSE;
 #endif
 
@@ -201,14 +198,14 @@ void renumber_from(linestruct *line)
 /* Display a warning about a key disabled in view mode. */
 void print_view_warning(void)
 {
-	statusbar(_("Key is invalid in view mode"));
+	statusline(AHEM, _("Key is invalid in view mode"));
 }
 
 /* When in restricted mode, show a warning and return TRUE. */
 bool in_restricted_mode(void)
 {
 	if (ISSET(RESTRICTED)) {
-		statusbar(_("This function is disabled in restricted mode"));
+		statusline(AHEM, _("This function is disabled in restricted mode"));
 		beep();
 		return TRUE;
 	} else
@@ -399,22 +396,28 @@ void window_init(void)
 		delwin(bottomwin);
 	}
 
+	topwin = NULL;
+
 	/* If the terminal is very flat, don't set up a title bar. */
 	if (LINES < 3) {
-		topwin = NULL;
 		editwinrows = 1;
 		/* Set up two subwindows.  If the terminal is just one line,
 		 * edit window and status-bar window will cover each other. */
 		edit = newwin(1, COLS, 0, 0);
 		bottomwin = newwin(1, COLS, LINES - 1, 0);
 	} else {
-		int toprows = (!ISSET(EMPTY_LINE) ? 1 : (LINES < 6) ? 1 : 2);
-		int bottomrows = (ISSET(NO_HELP) ? 1 : (LINES < 5) ? 1 : 3);
+		int toprows = ((ISSET(EMPTY_LINE) && LINES > 5) ? 2 : 1);
+		int bottomrows = ((ISSET(NO_HELP) || LINES < 5) ? 1 : 3);
 
+#ifndef NANO_TINY
+		if (ISSET(MINIBAR))
+			toprows = 0;
+#endif
 		editwinrows = LINES - toprows - bottomrows;
 
 		/* Set up the normal three subwindows. */
-		topwin = newwin(toprows, COLS, 0, 0);
+		if (toprows > 0)
+			topwin = newwin(toprows, COLS, 0, 0);
 		edit = newwin(editwinrows, COLS, toprows, 0);
 		bottomwin = newwin(bottomrows, COLS, toprows + editwinrows, 0);
 	}
@@ -422,13 +425,11 @@ void window_init(void)
 	/* In case the terminal shrunk, make sure the status line is clear. */
 	wipe_statusbar();
 
-#ifndef USE_SLANG
 	/* When not disabled, turn escape-sequence translation on. */
 	if (!ISSET(RAW_SEQUENCES)) {
 		keypad(edit, TRUE);
 		keypad(bottomwin, TRUE);
 	}
-#endif
 
 #ifdef ENABLED_WRAPORJUSTIFY
 	/* Set up the wrapping point, accounting for screen width when negative. */
@@ -526,10 +527,8 @@ void usage(void)
 	print_opt(_("-J <number>"), _("--guidestripe=<number>"),
 					N_("Show a guiding bar at this column"));
 #endif
-#ifndef USE_SLANG
 	print_opt("-K", "--rawsequences",
 					N_("Fix numeric keypad key confusion problem"));
-#endif
 #ifndef NANO_TINY
 	print_opt("-L", "--nonewlines",
 					N_("Don't add an automatic newline"));
@@ -643,6 +642,7 @@ void usage(void)
 		print_opt("-z", "--suspendable", N_("Enable suspension"));
 #ifndef NANO_TINY
 	print_opt("-%", "--stateflags", N_("Show some states on the title bar"));
+	print_opt("-_", "--minibar", N_("Show a feedback bar at the bottom"));
 #endif
 #ifdef HAVE_LIBMAGIC
 	print_opt("-!", "--magic", N_("Also try magic to determine syntax"));
@@ -659,8 +659,8 @@ void version(void)
 	printf(_(" GNU nano, version %s\n"), VERSION);
 #endif
 #ifndef NANO_TINY
-	printf(" (C) 1999-2011, 2013-2020 Free Software Foundation, Inc.\n");
-	printf(_(" (C) 2014-%s the contributors to nano\n"), "2020");
+	printf(" (C) 1999-2011, 2013-2021 Free Software Foundation, Inc.\n");
+	printf(_(" (C) 2014-%s the contributors to nano\n"), "2021");
 #endif
 	printf(_(" Compiled options:"));
 
@@ -775,9 +775,6 @@ void version(void)
 	printf(" --enable-utf8");
 #else
 	printf(" --disable-utf8");
-#endif
-#ifdef USE_SLANG
-	printf(" --with-slang");
 #endif
 	printf("\n");
 }
@@ -956,18 +953,11 @@ void do_suspend(int signal)
 /* Put nano to sleep (if suspension is enabled). */
 void do_suspend_void(void)
 {
-	if (ISSET(SUSPENDABLE)) {
-#ifdef USE_SLANG
-		selfinduced = TRUE;
-		do_suspend(0);
-		selfinduced = FALSE;
-#else
-		do_suspend(0);
-#endif
-	} else {
-		statusbar(_("Suspension is not enabled"));
+	if (!ISSET(SUSPENDABLE)) {
+		statusline(AHEM, _("Suspension is not enabled"));
 		beep();
-	}
+	} else
+		do_suspend(0);
 
 	ran_a_tool = TRUE;
 }
@@ -987,13 +977,9 @@ void do_continue(int signal)
 	/* Put the terminal in the desired state again. */
 	terminal_init();
 #endif
-#ifdef USE_SLANG
-	if (!selfinduced)
-		full_refresh();
-#else
+
 	/* Insert a fake keystroke, to neutralize a key-eating issue. */
 	ungetch(KEY_FLUSH);
-#endif
 }
 
 #if !defined(NANO_TINY) || defined(ENABLE_SPELLER) || defined(ENABLE_COLOR)
@@ -1021,14 +1007,6 @@ void handle_sigwinch(int signal)
 	the_window_resized = TRUE;
 }
 
-/* Compute and store how many extra rows each line needs when softwrapping. */
-void compute_the_extra_rows_per_line_from(linestruct *fromline)
-{
-	if (ISSET(SOFTWRAP))
-		for (linestruct *line = fromline; line != NULL; line = line->next)
-			line->extrarows = extra_chunks_in(line);
-}
-
 /* Reinitialize and redraw the screen completely. */
 void regenerate_screen(void)
 {
@@ -1071,7 +1049,6 @@ void regenerate_screen(void)
 
 	/* If we have an open buffer, redraw the contents of the subwindows. */
 	if (openfile) {
-		compute_the_extra_rows_per_line_from(openfile->filetop);
 		ensure_firstcolumn_is_aligned();
 		draw_all_subwindows();
 	}
@@ -1080,8 +1057,6 @@ void regenerate_screen(void)
 /* Handle the global toggle specified in flag. */
 void do_toggle(int flag)
 {
-	bool enabled;
-
 	if (flag == SUSPENDABLE && in_restricted_mode())
 		return;
 
@@ -1099,9 +1074,7 @@ void do_toggle(int flag)
 			break;
 #endif
 		case SOFTWRAP:
-			if (ISSET(SOFTWRAP))
-				compute_the_extra_rows_per_line_from(openfile->filetop);
-			else
+			if (!ISSET(SOFTWRAP))
 				openfile->firstcolumn = 0;
 			refresh_needed = TRUE;
 			break;
@@ -1117,17 +1090,27 @@ void do_toggle(int flag)
 #endif
 	}
 
-	if (ISSET(STATEFLAGS) && (flag == AUTOINDENT ||
-						flag == BREAK_LONG_LINES || flag == SOFTWRAP))
-		titlebar(NULL);
+	if (!ISSET(MINIBAR) && ISSET(STATEFLAGS))
+		if (flag == AUTOINDENT || flag == BREAK_LONG_LINES || flag == SOFTWRAP)
+			titlebar(NULL);
 
-	enabled = ISSET(flag);
+	if (ISSET(MINIBAR) && (flag == NO_HELP || flag == LINE_NUMBERS ))
+		return;
 
-	if (flag == NO_HELP || flag == NO_SYNTAX)
-		enabled = !enabled;
+	if (flag == CONSTANT_SHOW)
+		wipe_statusbar();
+	else if (!ISSET(MINIBAR) || !ISSET(STATEFLAGS) || flag == SMART_HOME ||
+						flag == NO_SYNTAX || flag == WHITESPACE_DISPLAY ||
+						flag == CUT_FROM_CURSOR || flag == TABS_TO_SPACES ||
+						flag == USE_MOUSE || flag == SUSPENDABLE) {
+		bool enabled = ISSET(flag);
 
-	statusline(HUSH, "%s %s", _(flagtostr(flag)),
-						enabled ? _("enabled") : _("disabled"));
+		if (flag == NO_HELP || flag == NO_SYNTAX)
+			enabled = !enabled;
+
+		statusline(REMARK, "%s %s", _(flagtostr(flag)),
+									enabled ? _("enabled") : _("disabled"));
+	}
 }
 #endif /* !NANO_TINY */
 
@@ -1202,36 +1185,15 @@ void enable_flow_control(void)
  * control characters. */
 void terminal_init(void)
 {
-#ifdef USE_SLANG
-	/* Slang curses emulation brain damage, part 2: Slang doesn't
-	 * implement raw(), nonl(), or noecho() properly, so there's no way
-	 * to properly reinitialize the terminal using them.  We have to
-	 * disable the special control keys and interpretation of the flow
-	 * control characters using termios, save the terminal state after
-	 * the first call, and restore it on subsequent calls. */
-	static struct termios desired_state;
-	static bool have_new_state = FALSE;
+	raw();
+	nonl();
+	noecho();
 
-	if (!have_new_state) {
-#endif
-		raw();
-		nonl();
-		noecho();
-		disable_extended_io();
-#ifdef USE_SLANG
-		tcgetattr(0, &desired_state);
-		have_new_state = TRUE;
-	} else
-		tcsetattr(0, TCSANOW, &desired_state);
+	disable_extended_io();
 
-	SLang_init_tty(-1, 0, 0);
-#endif
 	if (ISSET(PRESERVE))
 		enable_flow_control();
-#ifdef USE_SLANG
-	else
-		disable_flow_control();
-#endif
+
 	disable_kb_interrupt();
 
 #ifndef NANO_TINY
@@ -1268,14 +1230,15 @@ void confirm_margin(void)
 		needed_margin = 0;
 
 	if (needed_margin != margin) {
+		bool keep_focus = (margin > 0) && focusing;
+
 		margin = needed_margin;
 		editwincols = COLS - margin - thebar;
 
 #ifndef NANO_TINY
-		/* Recompute the softwrapped chunks for each line in the buffer,
-		 * and ensure a proper starting column for the first screen row. */
-		compute_the_extra_rows_per_line_from(openfile->filetop);
+		/* Ensure a proper starting column for the first screen row. */
 		ensure_firstcolumn_is_aligned();
+		focusing = keep_focus;
 #endif
 		/* The margin has changed -- schedule a full refresh. */
 		refresh_needed = TRUE;
@@ -1289,28 +1252,28 @@ void unbound_key(int code)
 	if (code == FOREIGN_SEQUENCE)
 		/* TRANSLATORS: This refers to a sequence of escape codes
 		 * (from the keyboard) that nano does not recognize. */
-		statusline(ALERT, _("Unknown sequence"));
+		statusline(AHEM, _("Unknown sequence"));
 	else if (code > 0x7F)
-		statusline(ALERT, _("Unbound key"));
+		statusline(AHEM, _("Unbound key"));
 	else if (meta_key) {
 #ifndef NANO_TINY
 		if (code < 0x20)
-			statusline(ALERT, _("Unbindable key: M-^%c"), code + 0x40);
+			statusline(AHEM, _("Unbindable key: M-^%c"), code + 0x40);
 		else
 #endif
 #ifdef ENABLE_NANORC
 		if (shifted_metas && 'A' <= code && code <= 'Z')
-			statusline(ALERT, _("Unbound key: Sh-M-%c"), code);
+			statusline(AHEM, _("Unbound key: Sh-M-%c"), code);
 		else
 #endif
-			statusline(ALERT, _("Unbound key: M-%c"), toupper(code));
+			statusline(AHEM, _("Unbound key: M-%c"), toupper(code));
 	} else if (code == ESC_CODE)
-		statusline(ALERT, _("Unbindable key: ^["));
+		statusline(AHEM, _("Unbindable key: ^["));
 	else if (code < 0x20)
-		statusline(ALERT, _("Unbound key: ^%c"), code + 0x40);
+		statusline(AHEM, _("Unbound key: ^%c"), code + 0x40);
 #if defined(ENABLE_BROWSER) || defined (ENABLE_HELP)
 	else
-		statusline(ALERT, _("Unbound key: %c"), code);
+		statusline(AHEM, _("Unbound key: %c"), code);
 #endif
 	set_blankdelay_to_one();
 }
@@ -1354,9 +1317,11 @@ int do_mouse(void)
 #ifndef NANO_TINY
 		/* Clicking where the cursor is toggles the mark, as does clicking
 		 * beyond the line length with the cursor at the end of the line. */
-		if (sameline && openfile->current_x == current_x_save)
+		if (sameline && openfile->current_x == current_x_save) {
 			do_mark();
-		else
+			if (ISSET(STATEFLAGS))
+				titlebar(NULL);
+		} else
 #endif
 			/* The cursor moved; clean the cutbuffer on the next cut. */
 			keep_cutbuffer = FALSE;
@@ -1437,12 +1402,13 @@ void inject(char *burst, size_t count)
 	linestruct *thisline = openfile->current;
 	size_t datalen = strlen(thisline->data);
 #ifndef NANO_TINY
-	size_t old_amount = openfile->current->extrarows;
 	size_t original_row = 0;
+	size_t old_amount = 0;
 
 	if (ISSET(SOFTWRAP)) {
 		if (openfile->current_y == editwinrows - 1)
 			original_row = chunk_for(xplustabs(), thisline);
+		old_amount = extra_chunks_in(thisline);
 	}
 #endif
 
@@ -1506,14 +1472,11 @@ void inject(char *burst, size_t count)
 	/* When softwrapping and the number of chunks in the current line changed,
 	 * or we were on the last row of the edit window and moved to a new chunk,
 	 * we need a full refresh. */
-	if (ISSET(SOFTWRAP)) {
-		openfile->current->extrarows = extra_chunks_in(openfile->current);
-		if (openfile->current->extrarows != old_amount ||
+	if (ISSET(SOFTWRAP) && (extra_chunks_in(openfile->current) != old_amount ||
 					(openfile->current_y == editwinrows - 1 &&
-					chunk_for(xplustabs(), openfile->current) > original_row)) {
-			refresh_needed = TRUE;
-			focusing = FALSE;
-		}
+					chunk_for(xplustabs(), openfile->current) > original_row))) {
+		refresh_needed = TRUE;
+		focusing = FALSE;
 	}
 #endif
 
@@ -1545,6 +1508,9 @@ void process_a_keystroke(void)
 	/* Read in a keystroke, and show the cursor while waiting. */
 	input = get_kbinput(edit, VISIBLE);
 
+	lastmessage = VACUUM;
+	hide_cursor = FALSE;
+
 #ifndef NANO_TINY
 	if (input == KEY_WINCH)
 		return;
@@ -1571,7 +1537,7 @@ void process_a_keystroke(void)
 			print_view_warning();
 		else {
 #ifndef NANO_TINY
-			if (openfile->mark && openfile->kind_of_mark == SOFTMARK) {
+			if (openfile->mark && openfile->softmark) {
 				openfile->mark = NULL;
 				refresh_needed = TRUE;
 			}
@@ -1645,7 +1611,7 @@ void process_a_keystroke(void)
 	if (shift_held && !openfile->mark) {
 		openfile->mark = openfile->current;
 		openfile->mark_x = openfile->current_x;
-		openfile->kind_of_mark = SOFTMARK;
+		openfile->softmark = TRUE;
 	}
 #endif
 
@@ -1657,7 +1623,7 @@ void process_a_keystroke(void)
 	 * discard a soft mark.  And when the marked region covers a
 	 * different set of lines, reset  the "last line too" flag. */
 	if (openfile->mark) {
-		if (!shift_held && openfile->kind_of_mark == SOFTMARK &&
+		if (!shift_held && openfile->softmark &&
 							(openfile->current != was_current ||
 							openfile->current_x != was_x ||
 							wanted_to_move(shortcut->func))) {
@@ -1711,9 +1677,7 @@ int main(int argc, char **argv)
 #ifdef ENABLE_NANORC
 		{"ignorercfiles", 0, NULL, 'I'},
 #endif
-#ifndef USE_SLANG
 		{"rawsequences", 0, NULL, 'K'},
-#endif
 #ifdef ENABLED_WRAPORJUSTIFY
 		{"trimblanks", 0, NULL, 'M'},
 #endif
@@ -1789,6 +1753,7 @@ int main(int argc, char **argv)
 		{"unix", 0, NULL, 'u'},
 		{"afterends", 0, NULL, 'y'},
 		{"stateflags", 0, NULL, '%'},
+		{"minibar", 0, NULL, '_'},
 #endif
 #ifdef HAVE_LIBMAGIC
 		{"magic", 0, NULL, '!'},
@@ -1812,15 +1777,10 @@ int main(int argc, char **argv)
 		fcntl(STDIN_FILENO, F_SETFL, stdin_flags & ~O_NONBLOCK);
 
 #ifdef ENABLE_UTF8
-	/* If setting the locale is successful and it uses UTF-8, we need
-	 * to use the multibyte functions for text processing. */
-	if (setlocale(LC_ALL, "") != NULL &&
-				strcmp(nl_langinfo(CODESET), "UTF-8") == 0) {
-#ifdef USE_SLANG
-		SLutf8_enable(1);
-#endif
+	/* If setting the locale is successful and it uses UTF-8, we will
+	 * need to use the multibyte functions for text processing. */
+	if (setlocale(LC_ALL, "") && strcmp(nl_langinfo(CODESET), "UTF-8") == 0)
 		utf8_init();
-	}
 #else
 	setlocale(LC_ALL, "");
 #endif
@@ -1830,25 +1790,15 @@ int main(int argc, char **argv)
 	textdomain(PACKAGE);
 #endif
 
-#if defined(ENABLE_UTF8) && !defined(NANO_TINY)
-	if (MB_CUR_MAX > MAXCHARLEN)
-		fprintf(stderr, "Unexpected large character size: %i bytes"
-						" -- please report a bug\n", (int)MB_CUR_MAX);
-#endif
-
-	/* Set sensible defaults, different from what Pico does. */
+	/* Set a sensible default, different from what Pico does. */
 	SET(NO_WRAP);
-	SET(SMOOTH_SCROLL);
-
-	/* Give a small visual hint that nano has changed. */
-	SET(MORE_SPACE);
 
 	/* If the executable's name starts with 'r', activate restricted mode. */
 	if (*(tail(argv[0])) == 'r')
 		SET(RESTRICTED);
 
 	while ((optchr = getopt_long(argc, argv, "ABC:DEFGHIJ:KLMNOPQ:RST:UVWX:Y:Z"
-				"abcdef:ghijklmno:pqr:s:tuvwxyz$%!", long_options, NULL)) != -1) {
+				"abcdef:ghijklmno:pqr:s:tuvwxyz$%_!", long_options, NULL)) != -1) {
 		switch (optchr) {
 #ifndef NANO_TINY
 			case 'A':
@@ -1898,11 +1848,9 @@ int main(int argc, char **argv)
 				}
 				break;
 #endif
-#ifndef USE_SLANG
 			case 'K':
 				SET(RAW_SEQUENCES);
 				break;
-#endif
 #ifndef NANO_TINY
 			case 'L':
 				SET(NO_NEWLINES);
@@ -2090,6 +2038,9 @@ int main(int argc, char **argv)
 			case '%':
 				SET(STATEFLAGS);
 				break;
+			case '_':
+				SET(MINIBAR);
+				break;
 #endif
 #ifdef HAVE_LIBMAGIC
 			case '!':
@@ -2195,15 +2146,14 @@ int main(int argc, char **argv)
 			free(alt_speller);
 			alt_speller = alt_speller_cmdline;
 		}
+		/* Strip leading whitespace from the speller command, if any. */
+		while (alt_speller && *alt_speller && isblank(*alt_speller))
+			memmove(alt_speller, alt_speller + 1, strlen(alt_speller));
 #endif
 
-		/* If an rcfile undid the default settings, copy it to the new flags. */
+		/* If an rcfile undid the default setting, copy it to the new flag. */
 		if (!ISSET(NO_WRAP))
 			SET(BREAK_LONG_LINES);
-		if (!ISSET(SMOOTH_SCROLL))
-			SET(JUMPY_SCROLLING);
-		if (!ISSET(MORE_SPACE))
-			SET(EMPTY_LINE);
 
 		/* Simply OR the boolean flags from rcfile and command line. */
 		for (size_t i = 0; i < sizeof(flags) / sizeof(flags[0]); i++)
@@ -2233,15 +2183,9 @@ int main(int argc, char **argv)
 #endif
 	}
 
-#ifdef USE_SLANG
-	/* When using Slang, do not let Slang translate escape sequences to
-	 * key codes, because it does it wrong for the longer sequences. */
-	SET(RAW_SEQUENCES);
-#else
 	/* When getting untranslated escape sequences, the mouse cannot be used. */
 	if (ISSET(RAW_SEQUENCES))
 		UNSET(USE_MOUSE);
-#endif
 
 #ifdef ENABLE_HISTORIES
 	/* Initialize the pointers for the Search/Replace/Execute histories. */
@@ -2353,6 +2297,8 @@ int main(int argc, char **argv)
 		interface_color_pair[GUIDE_STRIPE] = A_REVERSE;
 		interface_color_pair[SCROLL_BAR] = A_NORMAL;
 		interface_color_pair[SELECTED_TEXT] = hilite_attribute;
+		interface_color_pair[SPOTLIGHTED] = A_REVERSE;
+		interface_color_pair[PROMPT_BAR] = hilite_attribute;
 		interface_color_pair[STATUS_BAR] = hilite_attribute;
 		interface_color_pair[ERROR_MESSAGE] = hilite_attribute;
 		interface_color_pair[KEY_COMBO] = hilite_attribute;
@@ -2528,11 +2474,16 @@ int main(int argc, char **argv)
 #ifdef ENABLE_NANORC
 	if (startup_problem != NULL)
 		statusline(ALERT, startup_problem);
+
+#define NOTREBOUND  first_sc_for(MMAIN, do_help) && \
+						first_sc_for(MMAIN, do_help)->keycode == 0x07
+#else
+#define NOTREBOUND  TRUE
 #endif
 
 #ifdef ENABLE_HELP
 	if (*openfile->filename == '\0' && openfile->totsize == 0 &&
-				openfile->next == openfile && !ISSET(NO_HELP))
+				openfile->next == openfile && !ISSET(NO_HELP) && NOTREBOUND)
 		statusbar(_("Welcome to nano.  For basic help, type Ctrl+G."));
 #endif
 
@@ -2549,12 +2500,16 @@ int main(int argc, char **argv)
 		if (currmenu != MMAIN)
 			bottombars(MMAIN);
 
+#ifndef NANO_TINY
+		if (ISSET(MINIBAR) && lastmessage < REMARK)
+			minibar();
+		else
+#endif
 		/* Update the displayed current cursor position only when there
 		 * is no message and no keys are waiting in the input buffer. */
 		if (ISSET(CONSTANT_SHOW) && lastmessage == VACUUM && get_key_buffer_len() == 0)
 			report_cursor_position();
 
-		lastmessage = VACUUM;
 		as_an_at = TRUE;
 
 		/* Refresh just the cursor position or the entire edit window. */
@@ -2564,6 +2519,11 @@ int main(int argc, char **argv)
 		} else
 			edit_refresh();
 
+#ifndef NANO_TINY
+		/* Let the next keystroke cancel the highlighting of a search match. */
+		refresh_needed = spotlighted;
+		spotlighted = FALSE;
+#endif
 		errno = 0;
 		focusing = TRUE;
 
